==> aria2-1.6.3.tar.bz2: aria2-1.6.3/src/BtHaveAllMessage.cc <==
/* <!-- copyright */
/*
 * aria2 - The high speed download utility
 *
 * Copyright (C) 2006 Tatsuhiro Tsujikawa
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * In addition, as a special exception, the copyright holders give
 * permission to link the code of portions of this program with the
 * OpenSSL library under certain conditions as described in each
 * individual source file, and distribute linked combinations
 * including the two.
 * You must obey the GNU General Public License in all respects
 * for all of the code used other than OpenSSL.  If you modify
 * file(s) with this exception, you may extend this exception to your
 * version of the file(s), but you are not obligated to do so.  If you
 * do not wish to do so, delete this exception statement from your
 * version.  If you delete this exception statement from all source
 * files in the program, then also delete it here.
 */
/* copyright --> */
#include "BtHaveAllMessage.h"
#include "DlAbortEx.h"
#include "Peer.h"
#include "StringFormat.h"
#include "PieceStorage.h"
#include "message.h"

namespace aria2 {

const std::string BtHaveAllMessage::NAME("have all");

SharedHandle<BtHaveAllMessage> BtHaveAllMessage::create
(const unsigned char* data, size_t dataLength)
{
  return ZeroBtMessage::create<BtHaveAllMessage>(data, dataLength);
}

void BtHaveAllMessage::doReceivedAction()
{
  if(!peer->isFastExtensionEnabled()) {
    throw DL_ABORT_EX
      (StringFormat("%s received while fast extension is disabled",
		    toString().c_str()).str());
  }
  pieceStorage->subtractPieceStats(peer->getBitfield(),
				   peer->getBitfieldLength());
  peer->setAllBitfield();
  pieceStorage->addPieceStats(peer->getBitfield(), peer->getBitfieldLength());
  if(peer->isSeeder() && pieceStorage->downloadFinished()) {
    throw DL_ABORT_EX(MSG_GOOD_BYE_SEEDER);
  }
}

} // namespace aria2

==> aria2_1.12.0.orig.tar.bz2: aria2-1.12.0/src/BtHaveAllMessage.cc <==
/* <!-- copyright */
/*
 * aria2 - The high speed download utility
 *
 * Copyright (C) 2006 Tatsuhiro Tsujikawa
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * In addition, as a special exception, the copyright holders give
 * permission to link the code of portions of this program with the
 * OpenSSL library under certain conditions as described in each
 * individual source file, and distribute linked combinations
 * including the two.
 * You must obey the GNU General Public License in all respects
 * for all of the code used other than OpenSSL.  If you modify
 * file(s) with this exception, you may extend this exception to your
 * version of the file(s), but you are not obligated to do so.  If you
 * do not wish to do so, delete this exception statement from your
 * version.  If you delete this exception statement from all source
 * files in the program, then also delete it here.
 */
/* copyright --> */
#include "BtHaveAllMessage.h"
#include "DlAbortEx.h"
#include "Peer.h"
#include "fmt.h"
#include "PieceStorage.h"
#include "message.h"

namespace aria2 {

const std::string BtHaveAllMessage::NAME("have all");

BtHaveAllMessage::BtHaveAllMessage():ZeroBtMessage(ID, NAME) {}

SharedHandle<BtHaveAllMessage> BtHaveAllMessage::create
(const unsigned char* data, size_t dataLength)
{
  return ZeroBtMessage::create<BtHaveAllMessage>(data, dataLength);
}

void BtHaveAllMessage::doReceivedAction()
{
  if(!getPeer()->isFastExtensionEnabled()) {
    throw DL_ABORT_EX
      (fmt("%s received while fast extension is disabled",
           toString().c_str()));
  }
  if(isMetadataGetMode()) {
    return;
  }
  getPieceStorage()->subtractPieceStats(getPeer()->getBitfield(),
                                        getPeer()->getBitfieldLength());
  getPeer()->setAllBitfield();
  getPieceStorage()->addPieceStats
    (getPeer()->getBitfield(), getPeer()->getBitfieldLength());
  if(getPeer()->isSeeder() && getPieceStorage()->downloadFinished()) {
    throw DL_ABORT_EX(MSG_GOOD_BYE_SEEDER);
  }
}

} // namespace aria2
